/* GLSD
 *
 *  Authors: Eduardo Chappa <echappa@tamiu.edu>
 *	     Roberto Heredia <rheredia@tamiu.edu>
 *
 * Math Computations for chisquare distribution.
 */

#include <string.h>
#include <stdlib.h>
#include "glsd.h"
#include "chisquare.h"
#include "condition.h"
#include "math.h"


/* Code for computing critical values of the Chi Square distribution */
static GLSDVARS_S chisquarecv []= {
  {DUMMY_VAR, NULL, NULL,
        {NULL, NULL, INTEGER_TYPE, NULL}, null_condition, NULL},
  {DF_VAR, "Enter df value          : ", "df", 
        {NULL, NULL, INTEGER_TYPE, NULL}, df_condition, ERROR_DF},
  {P_VAR,  "Enter P  value          : ", "P",
        {NULL, NULL, DOUBLE_TYPE, NULL}, p_condition, ERROR_P},
  {LAST_VAR, NULL, NULL,
        {NULL, NULL, 0, NULL}, null_condition, NULL}
};


GLSDMETHOD_S chisquarecv_global = {
  "output critical values of the Chi Square distribution",
				/* name, used in output of results	*/
  "-cX",			/* command line switch for this method	*/
   4,				/* number of args from command line	*/
   chisquarecv_pick,		/* pick me, pick me!			*/
   chisquarecv_init,		/* init method - variables to read	*/
   copy_args,			/* load data to method			*/
   critical_chisquare,		/* process data				*/
   chisquarecv_print,		/* print result				*/
   chisquarecv_free,		/* free free set them free		*/
   NULL, 			/* spare function			*/
  (GLSDMETHOD_S *) NULL         /* next method				*/
};

GLSDMETHOD_S *
chisquarecv_pick(char *argv)
{
  GLSDMETHOD_S *rv; 
  rv = (argv != NULL && !strcmp(argv, chisquarecv_global.clswitch)) 
					? &chisquarecv_global : NULL;
  return rv;
}

void
chisquarecv_init(GLSDVARS_S **glsdv)
{
 *glsdv = (GLSDVARS_S *)chisquarecv;
}  

void *
critical_chisquare(GLSDVARS_S *stdv, int precision)
{
  GLSDRESULT_S *result;
  double p, max;
  int i, df;

  NUM_VAR(i,  stdv, DF_VAR);
  df = *(int *) ((RESULT_S *)stdv[i].varname.result)->result;

  NUM_VAR(i,  stdv, P_VAR);
  p = *(double *) ((RESULT_S *)stdv[i].varname.result)->result;

  /* find a reasonable maximum */
  for(max = 1.0; cumulative_X(max, df) < 1.0-p; max *=2.0);

  result = (GLSDRESULT_S *)fs_get(sizeof(GLSDRESULT_S));
  result->dval  = solve_chisquare(1.0-p, df, 0, max, precision);
  result->type  = DOUBLE_TYPE;
  result->precision = precision;
  return (void *)result;
}

void
chisquarecv_print(void *result)
{
 GLSDRESULT_S *rslt = (GLSDRESULT_S *)result;
 print_result("Critical chi-square", rslt->dval, rslt->precision);
}

void
chisquarecv_free(void **result)
{
  int i;
  fs_give(result);
  for(i = 0; chisquarecv[i].type != LAST_VAR; i++)
     if(chisquarecv[i].varname.name && *chisquarecv[i].varname.name){
        fs_give((void **)&chisquarecv[i].varname.name);
	fs_give(&((RESULT_S *)chisquarecv[i].varname.result)->result);
	fs_give(&chisquarecv[i].varname.result);
     }  
}


/* Code for computing  values of the Chi Square distribution */
static GLSDVARS_S chisquarev []= {
  {DUMMY_VAR, NULL, NULL,
        {NULL, NULL, INTEGER_TYPE, NULL}, null_condition, NULL},
  {DF_VAR, "Enter df value          : ", "df", 
        {NULL, NULL, INTEGER_TYPE, NULL}, df_condition, ERROR_DF},
  {P_VAR,  "Enter X  value          : ", "X",
        {NULL, NULL, DOUBLE_TYPE, NULL}, null_condition, NULL},
  {LAST_VAR, NULL, NULL,
        {NULL, NULL, 0, NULL}, null_condition, NULL}
};


GLSDMETHOD_S chisquare_global = {
  "output values of the Chi Square distribution",
				/* name, used in output of results	*/
  "-X",				/* command line switch for this method	*/
   4,				/* number of args from command line	*/
   chisquare_pick,		/* pick me, pick me!			*/
   chisquare_init,		/* init method - variables to read	*/
   copy_args,			/* load data to method			*/
   compute_chisquare,		/* process data				*/
   chisquare_print,		/* print result				*/
   chisquare_free,		/* free free set them free		*/
   NULL, 			/* spare function			*/
  (GLSDMETHOD_S *) NULL         /* next method				*/
};

GLSDMETHOD_S *
chisquare_pick(char *argv)
{
  GLSDMETHOD_S *rv; 
  rv = (argv != NULL && !strcmp(argv, chisquare_global.clswitch)) 
					? &chisquare_global : NULL;
  return rv;
}

void
chisquare_init(GLSDVARS_S **glsdv)
{
 *glsdv = (GLSDVARS_S *)chisquarev;
}  

void *
compute_chisquare(GLSDVARS_S *stdv, int precision)
{
  GLSDRESULT_S *result;
  double p;
  int i, df;

  NUM_VAR(i,  stdv, DF_VAR);
  df = *(int *) ((RESULT_S *)stdv[i].varname.result)->result;

  NUM_VAR(i,  stdv, P_VAR);
  p = *(double *) ((RESULT_S *)stdv[i].varname.result)->result;

  /* find a reasonable maximum */
  result = (GLSDRESULT_S *)fs_get(sizeof(GLSDRESULT_S));
  result->dval  = cumulative_X(p, df);
  result->type  = DOUBLE_TYPE;
  result->precision = precision;
  return (void *)result;
}

void
chisquare_print(void *result)
{
 GLSDRESULT_S *rslt = (GLSDRESULT_S *)result;
 print_result("Area for chi-square", rslt->dval, rslt->precision);
}

void
chisquare_free(void **result)
{
  int i;
  fs_give(result);
  for(i = 0; chisquarev[i].type != LAST_VAR; i++)
     if(chisquarev[i].varname.name && *chisquarev[i].varname.name){
        fs_give((void **)&chisquarev[i].varname.name);
	fs_give(&((RESULT_S *)chisquarev[i].varname.result)->result);
	fs_give(&chisquarev[i].varname.result);
     }  
}
