/* GLSD
 *
 *  Authors: Eduardo Chappa <echappa@tamiu.edu>
 *	     Roberto Heredia <rheredia@tamiu.edu>
 *
 * Math Computations for Snedecor's F distribution.
 */

#include <string.h>
#include <stdlib.h>
#include "glsd.h"
#include "condition.h"
#include "math.h"
#include "fisher.h"


/* Code for computing critical values of the Fisher-Snedecor distribution */
static GLSDVARS_S fishercv []= {
  {DUMMY_VAR, NULL, NULL,
        {NULL, NULL, INTEGER_TYPE, NULL}, null_condition, NULL},
  {DF_VAR, "Enter df1 value         : ", "df1", 
        {NULL, NULL, INTEGER_TYPE, NULL}, df_condition, ERROR_DF},
  {DF2_VAR, "Enter df2 value         : ", "df2", 
        {NULL, NULL, INTEGER_TYPE, NULL}, df_condition, ERROR_DF},
  {P_VAR,  "Enter P  value          : ", "P",
        {NULL, NULL, DOUBLE_TYPE, NULL}, p_condition, ERROR_P},
  {LAST_VAR, NULL, NULL,
        {NULL, NULL, 0, NULL}, null_condition, NULL}
};


GLSDMETHOD_S fishercv_global = {
  "output critical values of the Fisher-Snedecor distribution",
				/* name, used in output of results	*/
  "-cF",			/* command line switch for this method	*/
   5,				/* number of args from command line	*/
   fishercv_pick,		/* pick me, pick me!			*/
   fishercv_init,		/* init method - variables to read	*/
   copy_args,			/* load data to method			*/
   critical_fisher,		/* process data				*/
   fishercv_print,		/* print result				*/
   fishercv_free,		/* free free set them free		*/
   NULL, 			/* spare function			*/
  (GLSDMETHOD_S *) NULL         /* next method				*/
};

GLSDMETHOD_S *
fishercv_pick(char *argv)
{
  GLSDMETHOD_S *rv; 
  rv = (argv != NULL && !strcmp(argv, fishercv_global.clswitch)) 
					? &fishercv_global : NULL;
  return rv;
}

void
fishercv_init(GLSDVARS_S **glsdv)
{
 *glsdv = (GLSDVARS_S *) fishercv;
}  

void *
critical_fisher(GLSDVARS_S *stdv, int precision)
{
  GLSDRESULT_S *result;
  double p, max;
  int i, df, df2;

  NUM_VAR(i,  stdv, DF_VAR);
  df = *(int *) ((RESULT_S *)stdv[i].varname.result)->result;

  NUM_VAR(i,  stdv, DF2_VAR);
  df2 = *(int *) ((RESULT_S *)stdv[i].varname.result)->result;

  NUM_VAR(i,  stdv, P_VAR);
  p = *(double *) ((RESULT_S *)stdv[i].varname.result)->result;

  /* find a reasonable maximum */
  for(max = 1.0; cumulative_F(max, df, df2) < 1.0 - p; max *=2.0);

  result = (GLSDRESULT_S *)fs_get(sizeof(GLSDRESULT_S));
  result->dval  = solve_fisher(1.0-p, df, df2, 0, max, precision);
  result->type  = DOUBLE_TYPE;
  result->precision = precision;
  return (void *)result;
}

void
fishercv_print(void *result)
{
 GLSDRESULT_S *rslt = (GLSDRESULT_S *)result;
 print_result("Critical F", rslt->dval, rslt->precision);
}

void
fishercv_free(void **result)
{
  int i;
  fs_give(result);
  for(i = 0; fishercv[i].type != LAST_VAR; i++)
     if(fishercv[i].varname.name && *fishercv[i].varname.name){
        fs_give((void **)&fishercv[i].varname.name);
	fs_give(&((RESULT_S *)fishercv[i].varname.result)->result);
	fs_give(&fishercv[i].varname.result);
     }  
}


/* Code for computing  values of the Snedecor distribution */
static GLSDVARS_S fisherv []= {
  {DUMMY_VAR, NULL, NULL,
        {NULL, NULL, INTEGER_TYPE, NULL}, null_condition, NULL},
  {DF_VAR,  "Enter df1 value         : ", "df1", 
        {NULL, NULL, INTEGER_TYPE, NULL}, df_condition, ERROR_DF},
  {DF2_VAR, "Enter df2 value         : ", "df2", 
        {NULL, NULL, INTEGER_TYPE, NULL}, df_condition, ERROR_DF},
  {P_VAR,   "Enter F value           : ", "F",
        {NULL, NULL, DOUBLE_TYPE, NULL}, null_condition, NULL},
  {LAST_VAR, NULL, NULL,
        {NULL, NULL, 0, NULL}, null_condition, NULL}
};


GLSDMETHOD_S fisher_global = {
  "output values of the Fisher-Snedecor distribution",
				/* name, used in output of results	*/
  "-F",				/* command line switch for this method	*/
   5,				/* number of args from command line	*/
   fisher_pick,			/* pick me, pick me!			*/
   fisher_init,			/* init method - variables to read	*/
   copy_args,			/* load data to method			*/
   compute_fisher,		/* process data				*/
   fisher_print,		/* print result				*/
   fisher_free,			/* free free set them free		*/
   NULL, 			/* spare function			*/
  (GLSDMETHOD_S *) NULL         /* next method				*/
};

GLSDMETHOD_S *
fisher_pick(char *argv)
{
  GLSDMETHOD_S *rv; 
  rv = (argv != NULL && !strcmp(argv, fisher_global.clswitch)) 
					? &fisher_global : NULL;
  return rv;
}

void
fisher_init(GLSDVARS_S **glsdv)
{
 *glsdv = (GLSDVARS_S *)fisherv;
}  

void *
compute_fisher(GLSDVARS_S *stdv, int precision)
{
  GLSDRESULT_S *result;
  double p;
  int i, df, df2;

  NUM_VAR(i,  stdv, DF_VAR);
  df = *(int *) ((RESULT_S *)stdv[i].varname.result)->result;

  NUM_VAR(i,  stdv, DF2_VAR);
  df2 = *(int *) ((RESULT_S *)stdv[i].varname.result)->result;

  NUM_VAR(i,  stdv, P_VAR);
  p = *(double *) ((RESULT_S *)stdv[i].varname.result)->result;

  /* find a reasonable maximum */
  result = (GLSDRESULT_S *)fs_get(sizeof(GLSDRESULT_S));
  result->dval  = cumulative_F(p, df, df2);
  result->type  = DOUBLE_TYPE;
  result->precision = precision;
  return (void *)result;
}

void
fisher_print(void *result)
{
 GLSDRESULT_S *rslt = (GLSDRESULT_S *)result;
 print_result("Area for Snedecor-Fisher", rslt->dval, rslt->precision);
}

void
fisher_free(void **result)
{
  int i;
  fs_give(result);
  for(i = 0; fisherv[i].type != LAST_VAR; i++)
     if(fisherv[i].varname.name && *fisherv[i].varname.name){
        fs_give((void **)&fisherv[i].varname.name);
	fs_give(&((RESULT_S *)fisherv[i].varname.result)->result);
	fs_give(&fisherv[i].varname.result);
     }  
}
